<?php
class Xoogu_GeoText
{
    
    //Options for this plugin
    protected $_aOptions = array();
    //plugin version
    public static $sVersion = '2016-04-26';
	protected $_sCountryCode;
	protected $_aGroupings;
    
    /**
     *Initialise the plugin
     */
    public function __construct()
	{
		//get the options from the db
		$this->_aOptions = get_site_option('xoogu_geotext');
		
		//register the shortcode handlers
		add_shortcode('geoText', array($this,'geoText'));
		add_shortcode('geoContent', array($this,'geoContent'));
    }
	
	
	/**
	 *Gets the 2 digit ISO country code of the visitor based on their IP address
	 *@return string The 2 digit ISO country code
	 */
	public function getCountryCode()
	{
		if ($this->_sCountryCode) {
			return $this->_sCountryCode;
		}
		
		//get the user's IP
		if (isset($_SERVER['HTTP_X_REAL_IP'])) {
			$sIP = $_SERVER['HTTP_X_REAL_IP'];
		} elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
			$sIP = $_SERVER['HTTP_X_FORWARDED_FOR'];
		} else {
			$sIP = $_SERVER['REMOTE_ADDR'];
		}
        
		//declare the countryCode var
		$sCountryCode = '';
		
        //get the country code from the IP based on the method specified in the settings
        if (!empty($this->_aOptions['geoFunc'])) {
			switch ($this->_aOptions['geoFunc']) {
				case 'server' :
					$sCountryCode = $_SERVER['GEOIP_COUNTRY_CODE'];
					break;
				case 'php' :
					//PHP GeoIP extension - http://www.php.net/manual/en/function.geoip-country-code-by-name.php
					if (function_exists('geoip_country_code_by_name')) {
						$sCountryCode = geoip_country_code_by_name($sIP);
					}
					break;
				case 'geoip-detect2' :
					//GeoIP Detection plugin - https://wordpress.org/plugins/geoip-detect/
					if (function_exists('geoip_detect2_get_info_from_current_ip')) {
						if ( ($oUserInfo = geoip_detect2_get_info_from_current_ip()) ) {
							$sCountryCode = $oUserInfo->country->isoCode;
						}
					}
					break;
				case 'ip2nation' :
					//Try ip2nation table - https://wordpress.org/plugins/wp-ip2nation-installer/
					global $wpdb;
					if ($wpdb->get_var("SHOW TABLES LIKE 'ip2nation'")) {
						//ip2nation uses code of 'UK' for GB in main table. In ip2nationCountries the iso_code_2 is correct (GB), but Ukraine is given as UK (should be UA)
						$sql = $wpdb->prepare('SELECT country FROM ip2nation WHERE ip < INET_ATON(%s) ORDER BY ip DESC LIMIT 0,1', $sIP);
						$sCountryCode = $wpdb->get_var($sql);
					}
					break;
			}
		}
		
		$sCountryCode = apply_filters('countryCode', $sCountryCode);
		
        if(empty($sCountryCode)){
            return;
        }
		
		return $this->_sCountryCode = strtolower($sCountryCode);
	}
	
	
	/**
	 *Gets the groupings that the user's country belongs to
	 *@return array groupings that the user's country belongs to
	 */
	public function getGroupings()
	{
		if ($this->_aGroupings) {
			return $this->_aGroupings;
		}
		
		global $wpdb;
		$sql = $wpdb->prepare('SELECT `grouping` FROM `'.$wpdb->base_prefix.'xoogu_geoGroups` WHERE `country` = %s', $this->_sCountryCode);
		$aGroupings = $wpdb->get_results($sql, ARRAY_N);
		//convert the results so that each 'row' is just the fetched value rather than an array containing the fetched value
		foreach ($aGroupings as &$grouping) {
			$grouping = $grouping[0];
		}
		unset($grouping);
		
		return $this->_aGroupings = $aGroupings;
	}
	
	
	/**
	 *Returns either the supplied $content, or country / grouping specific content specified in
	 * the attributes if the user's country matches
	 *@param array $attrs Array of shortcode attributes, in the form of:
	 * countryCode / grouping => content to show for this country / grouping
	 *@param string $content The $content to be returned if the user's country does not match
	 * any of the countries or groupings given in the shortcode attributes
	 *@return string Either the unmodified $content or the content from the matching shortcode
	 * attribute
	 */
	public function geoText($attrs, $content)
	{
		$sCountryCode = $this->getCountryCode();
		if (isset($attrs[$sCountryCode])) {
			//if the country is included, then display the content specified in that attribute
			return $attrs[$sCountryCode];
		}
		//get what groupings the country is in
		$aGroupings = $this->getGroupings();
		foreach ($aGroupings as $sGrouping) {
			if (isset($attrs[$sGrouping])) {
				//if a grouping that contains the country is included, then display the content specified in that attribute
				return $attrs[$sGrouping];
			}
		}
		//if the country or any grouping that contains it was not listed, then display the content as-is
		return do_shortcode($content);
	}
	
	
	/**
	 *Returns the supplied $content only if the user's country is listed in the $attrs
	 *@param array $attrs Array of shortcode attributes, can be one or more of:
	 *	exclude => string List of countries (comma separated) that should not see the $content
	 *	countries => string List of countries (comma separated) that should see the $content
	 *	groupings => string List of groupings that should see the $content
	 *@param string $content The $content to be returned or discarded depending on whether
	 *	the user's country matches that given in the shortcode attributes
	 *@return string Either the $content or an empty string
	 */
	public function geoContent($attrs, $content)
	{
		$countryCode = $this->getCountryCode();
		if (!empty($attrs['excludecountries'])) {
			if (stristr($attrs['excludecountries'], $countryCode)) {
				//if the country is excluded, then don't display the content
				return '';
			} elseif (empty($attrs['countries']) && empty($attrs['groupings']) && empty($attrs['excludegroupings'])) {
				//if only certain countries are excluded and none are specifically included
				//but the user's country doesn't match those excluded, then show the content
				return do_shortcode($content);
			}
		}
		if (!empty($attrs['countries']) && stristr($attrs['countries'], $countryCode)) {
			//if the country is included, then display the content
			return do_shortcode($content);
		}
		if(!empty($attrs['excludegroupings'])) {
			//get what groupings the country is in
			$aGroups = $this->getGroupings();
			foreach ($aGroups as $sGroup) {
				if (preg_match('/\b'.$sGroup.'\b/i', $attrs['excludegroupings'])) {
					//if a grouping that contains the country is included, then display the content
					return '';
				}
			}
			if (empty($attrs['countries']) && empty($attrs['groupings'])) {
				//if only certain groupings are excluded and none are specifically included
				//but the user's grouping doesn't match those excluded, then show the content
				return do_shortcode($content);
			}
		}
		if(!empty($attrs['groupings'])) {
			//get what groupings the country is in
			$aGroups = $this->getGroupings();
			foreach ($aGroups as $sGroup) {
				if (preg_match('/\b'.$sGroup.'\b/i', $attrs['groupings'])) {
					//if a grouping that contains the country is included, then display the content
					return do_shortcode($content);
				}
			}
		}
		//If the country was not included either by itself or as part of a grouping, then don't display the content
		return '';
	}
	
}
